 
/************************************************************************
 *
 * Created with Falch.net DeveloperStudio
 * http://www.falch.net/
 * 
 * File : frmMain.c
 * 
 * Description :  
 *
 * History:
 *			Name		Date					Description
 *			----		----					-----------
 *			Gollub	23.01.2002 19:18:17		Created

 *
 ***********************************************************************/

#include <PalmOS.h>
#include "Serial.h"
#include "Serial_res.h"   

#define UART_MISC_REG_ADDR		0xFFFFF908
#define UMISC_RTS_BIT			(UInt16)0x0040
#define UMISC_RTSCONT_BIT		(UInt16)0x0080
#define InitialBaudrate			9600
#define InitialBaudrateButton	pbnBaud9600	

UInt16	PortID;	
Boolean	PortOpen = false;

void SetRTSState (Boolean State)
{
	if (PortOpen)
	{
		Err     Error;
		UInt16* ControlRegister;
		UInt16  Value;
		UInt32  ProcessorID, ProcessorType;

		// we __MUST__ check the CPU type before meddling with the proprietary CPU registers
		Error = FtrGet(sysFtrCreator, sysFtrNumProcessorID, &ProcessorID);
		if (Error == 0)
		{
			ProcessorType = ProcessorID & sysFtrNumProcessorMask;
			if ((ProcessorType == sysFtrNumProcessor328) || (ProcessorType == sysFtrNumProcessorEZ))
			{		
				// read UART misc. register
				ControlRegister = (UInt16*)UART_MISC_REG_ADDR;
				Value = *ControlRegister;
	
				// we need to make sure the RTSCONTROL bit is cleared, because otherwise the UART
				// will control the RTS line automatically, ignoring our attempts to set it
				Value &= ~UMISC_RTSCONT_BIT;
	
				// set RTS bit
				if (State)
					Value |= UMISC_RTS_BIT;
				else
					Value &= ~UMISC_RTS_BIT;
	
				// write UART misc. register
				*ControlRegister = Value;
			}
		}
	}	
}

void OpenPort()
{
	Err	Error;
	Error = SrmOpen(serPortCradlePort, InitialBaudrate, &PortID);
	if (Error == 0)
	{
		UInt32	Flags = srmSettingsFlagStopBits1 | srmSettingsFlagBitsPerChar8; 
		UInt16	FlagsSize = sizeof(UInt32);

		SrmControl(PortID,srmCtlSetFlags,&Flags,&FlagsSize);		
		PortOpen = true;
		SetRTSState(false);
	} 
}

void ClosePort()
{
	if (PortOpen)
		SrmClose(PortID);
}

void SetBaudRate (Int32 BaudRate)
{
	if (PortOpen)
	{
		UInt16 BaudRateSize = sizeof(Int32);
		SrmControl(PortID,srmCtlSetBaudRate,&BaudRate,&BaudRateSize);		
	}
}

void SetPushButtonState(UInt16 PushButtonID, Boolean State)
{
	FormPtr 		Form;
	ControlType*	PushButton;
	UInt16			PushButtonIndex;

	Form = FrmGetFormPtr(frmMain);
	PushButtonIndex = FrmGetObjectIndex(Form, PushButtonID);
	PushButton = FrmGetObjectPtr(Form, PushButtonIndex);
	CtlSetValue(PushButton, State);
}

Boolean GetPushButtonSelected(UInt16 PushButtonID)
{
	FormPtr 		Form;
	ControlType*	PushButton;
	UInt16			PushButtonIndex;

	Form = FrmGetFormPtr(frmMain);
	PushButtonIndex = FrmGetObjectIndex(Form, PushButtonID);
	PushButton = FrmGetObjectPtr(Form, PushButtonIndex);
	return CtlGetValue(PushButton);	
}

void DisplayReceivedByte(UInt8 ReceivedByte)
{
	SetPushButtonState(pbnResceiveBit7_1,ReceivedByte & 0x80);
	SetPushButtonState(pbnResceiveBit7_0,!(ReceivedByte & 0x80));

	SetPushButtonState(pbnResceiveBit6_1,ReceivedByte & 0x40);
	SetPushButtonState(pbnResceiveBit6_0,!(ReceivedByte & 0x40));	

	SetPushButtonState(pbnResceiveBit5_1,ReceivedByte & 0x20);
	SetPushButtonState(pbnResceiveBit5_0,!(ReceivedByte & 0x20));	

	SetPushButtonState(pbnResceiveBit4_1,ReceivedByte & 0x10);
	SetPushButtonState(pbnResceiveBit4_0,!(ReceivedByte & 0x10));	

	SetPushButtonState(pbnResceiveBit3_1,ReceivedByte & 0x08);
	SetPushButtonState(pbnResceiveBit3_0,!(ReceivedByte & 0x08));	

	SetPushButtonState(pbnResceiveBit2_1,ReceivedByte & 0x04);
	SetPushButtonState(pbnResceiveBit2_0,!(ReceivedByte & 0x04));	

	SetPushButtonState(pbnResceiveBit1_1,ReceivedByte & 0x02);
	SetPushButtonState(pbnResceiveBit1_0,!(ReceivedByte & 0x02));	

	SetPushButtonState(pbnResceiveBit0_1,ReceivedByte & 0x01);
	SetPushButtonState(pbnResceiveBit0_0,!(ReceivedByte & 0x01));		
}

void SelectPushButtons()
{
	SetPushButtonState(InitialBaudrateButton,true);
	SetPushButtonState(pbnSendBit7_0,true);
	SetPushButtonState(pbnSendBit6_0,true);
	SetPushButtonState(pbnSendBit5_0,true);
	SetPushButtonState(pbnSendBit4_0,true);
	SetPushButtonState(pbnSendBit3_0,true);
	SetPushButtonState(pbnSendBit2_0,true);
	SetPushButtonState(pbnSendBit1_0,true);
	SetPushButtonState(pbnSendBit0_0,true);
	SetPushButtonState(pbnRTS0,true);
	SetPushButtonState(pbnCTS0,true);
	DisplayReceivedByte(0);	
}

void SendByte()
{
	if (PortOpen)		
	{
		Err		Error;
		UInt8	ByteToSend = 0;
		
		if (GetPushButtonSelected(pbnSendBit7_1))
			ByteToSend |= 0x80;

		if (GetPushButtonSelected(pbnSendBit6_1))
			ByteToSend |= 0x40;

		if (GetPushButtonSelected(pbnSendBit5_1))
			ByteToSend |= 0x20;

		if (GetPushButtonSelected(pbnSendBit4_1))
			ByteToSend |= 0x10;

		if (GetPushButtonSelected(pbnSendBit3_1))
			ByteToSend |= 0x08;

		if (GetPushButtonSelected(pbnSendBit2_1))
			ByteToSend |= 0x04;
		
		if (GetPushButtonSelected(pbnSendBit1_1))
			ByteToSend |= 0x02;

		if (GetPushButtonSelected(pbnSendBit0_1))
			ByteToSend |= 0x01;	
		
		SrmSend(PortID, &ByteToSend, 1, &Error);
	}		 
}

void ResceiveByte()
{
	if (PortOpen)
	{
		Err		Error;
		UInt32	ByteCount;
		UInt8	ReceivedByte;
		
		SrmReceiveCheck(PortID,&ByteCount);
		if (ByteCount>=1)
		{
			SrmReceive(PortID,&ReceivedByte,1,0,&Error);
			DisplayReceivedByte(ReceivedByte);					
		} 	
	}
}

void UpdateCTS()
{
	if (PortOpen)
	{
		UInt32	StatusField;
		UInt16	LineErrors;
	
		SrmGetStatus(PortID,&StatusField,&LineErrors);
		SetPushButtonState(pbnCTS1,StatusField & srmStatusCtsOn);
		SetPushButtonState(pbnCTS0,!(StatusField & srmStatusCtsOn));	
	}	
}

static Boolean frmMain_pbnBaud300_OnSelect(EventPtr event)
{
	SetBaudRate(300);
	return true;
}

static Boolean frmMain_pbnBaud600_OnSelect(EventPtr event)
{
	SetBaudRate(600);
	return true;
}

static Boolean frmMain_pbnBaud1200_OnSelect(EventPtr event)
{
	SetBaudRate(1200);
	return true;
}

static Boolean frmMain_pbnBaud2400_OnSelect(EventPtr event)
{
	SetBaudRate(2400);
	return true;
}

static Boolean frmMain_pbnBaud4800_OnSelect(EventPtr event)
{
	SetBaudRate(4800);
	return true;
}

static Boolean frmMain_pbnBaud9600_OnSelect(EventPtr event)
{
	SetBaudRate(9600);
	return true;
}

static Boolean frmMain_pbnBaud19k2_OnSelect(EventPtr event)
{
	SetBaudRate(19200);
	return true;
}

static Boolean frmMain_pbnBaud38k4_OnSelect(EventPtr event)
{
	SetBaudRate(38400);
	return true;
}

static Boolean frmMain_pbnBaud57k6_OnSelect(EventPtr event)
{
	SetBaudRate(57600);
	return true;
}

static Boolean frmMain_pbnBaud115k2_OnSelect(EventPtr event)
{
	SetBaudRate(115200);
	return true;
}

static Boolean frmMain_pbnSend_OnSelect(EventPtr event)
{
	SendByte();
	return true;
}

static Boolean frmMain_pbnRTS1_OnSelect(EventPtr event)
{
	SetRTSState(true);
	return true;
}

static Boolean frmMain_pbnRTS0_OnSelect(EventPtr event)
{
	SetRTSState(false);
	return true;
}

/***********************************************************************
 *
 * FUNCTION:	frmMain_HandleEvent
 *
 * DESCRIPTION:	Handles a Form event
 *
 * PARAMETERS:	event	pointer to an event structure
 *					
 * RETURNED:	returns handled/not handled
 *
 * REVISION HISTORY:
 *			Name		Date					Description
 *			----		----					-----------
 *			Gollub	23.01.2002 19:18:17		Created

 * 
 ***********************************************************************/

Boolean frmMain_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				// pbnBaud300 receives an event
				case pbnBaud300:
					handled = frmMain_pbnBaud300_OnSelect(event);
					break;
				// pbnBaud600 receives an event
				case pbnBaud600:
					handled = frmMain_pbnBaud600_OnSelect(event);
					break;
				// pbnBaud1200 receives an event
				case pbnBaud1200:
					handled = frmMain_pbnBaud1200_OnSelect(event);
					break;
				// pbnBaud2400 receives an event
				case pbnBaud2400:
					handled = frmMain_pbnBaud2400_OnSelect(event);
					break;
				// pbnBaud4800 receives an event
				case pbnBaud4800:
					handled = frmMain_pbnBaud4800_OnSelect(event);
					break;
				// pbnBaud9600 receives an event
				case pbnBaud9600:
					handled = frmMain_pbnBaud9600_OnSelect(event);
					break;
				// pbnBaud19k2 receives an event
				case pbnBaud19k2:
					handled = frmMain_pbnBaud19k2_OnSelect(event);
					break;
				// pbnBaud38k4 receives an event
				case pbnBaud38k4:
					handled = frmMain_pbnBaud38k4_OnSelect(event);
					break;
				// pbnBaud57k6 receives an event
				case pbnBaud57k6:
					handled = frmMain_pbnBaud57k6_OnSelect(event);
					break;
				// pbnBaud115k2 receives an event
				case pbnBaud115k2:
					handled = frmMain_pbnBaud115k2_OnSelect(event);
					break;				
				// pbnSend receives an event
				case pbnSend:
					handled = frmMain_pbnSend_OnSelect(event);
					break;
				// pbnRTS1 receives an event
				case pbnRTS1:
					handled = frmMain_pbnRTS1_OnSelect(event);
					break;
				// pbnRTS0 receives an event
				case pbnRTS0:
					handled = frmMain_pbnRTS0_OnSelect(event);
					break;
			}
			break;
		case frmOpenEvent:
			// Repaint form on open
			form = FrmGetActiveForm();
			FrmDrawForm(form);

			SelectPushButtons();

			handled = true;			
			break;		
		default:
			break;
	}

	return handled;
}
